#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "diff.h"

#if defined FORTIFY
# include "fortify.h"
#endif

#define OPT_FLAG  '-'		/* command line option switch recogniser */
#define MAXLINE  256		/* size of input buffer, but a line can be of any length */

#if !defined FALSE
# define FALSE 0
#endif

#if !defined TRUE
# define TRUE 1
#endif

typedef char *char_ptr;
typedef FILE *FILE_PTR;

static char xx1[132], xx2[132];	/* space for file names */

static char_ptr infile_name[3] = {NULL,xx1,xx2};
static FILE_PTR infile[3];		/* input file pointers */

static int bar_col = -1;
static int top_skip = -1;
static int bot_skip = -1;
static int page_len = -1;
static int up_case = -1;
static int re_sync = -1;
static int blanks = -1;
static int scr_on = 1;
static int skip1 = -1;
static int skip2 = -1;
static int onlylinenrs = -1;
static int trace_enabled = -1;
static int debug = -1;

static int output = 0;			/* boolean - is change barred outfile on */
static int files = 0;			/* nr of files in command line */
static char outfile_name[132];		/* output file name */
static int command_errors = 0;		/* number of command line errors */

static FILE *outfile;			/* changebarred output file ptr */

/* strip_opt - processes each command line option */
static void strip_opt (ac,av)
	int ac;
	char *av[];
{
	int i;

	for (i=1; i<ac; i++) {
		if (av[i][0] == OPT_FLAG) {
			switch (av[i][1]) {
				case 'b': bar_col = atoi (av[++i]);
					  break;
				case 'h': top_skip = atoi (av[++i]);
					  break;
				case 'f': bot_skip = atoi (av[++i]);
					  break;
				case 'p': page_len = atoi (av[++i]);
					  break;
				case 'c': up_case = 1;
					  break;
				case 'r': re_sync = atoi (av[++i]);
					  break;
				case 'w': blanks = 1;
					  break;
				case 's': scr_on = 0;
					  break;
				case 'n': skip1 = skip2 = atoi (av[++i]);
					  break;
				case 'o': skip2 = atoi (av[++i]);
					  break;
				case 'l': onlylinenrs = TRUE;
					  break;
#ifdef TRACER_FUNCTIONS
				case 't': trace_enabled = debug = 1;
					  break;
#endif
				default:  printf ("Unrecognised option %s\n",av[i]);
					  command_errors++;
			} /* switch av[i][1] */
		}
		else {
			switch (files) {
				case 0:	strcpy (infile_name[1], av[i]);
					break;
				case 1: strcpy (infile_name[2], av[i]);
					break;
				case 2: strcpy (outfile_name, av[i]);
					output = 1;
					break;
				default:
					printf ("Error: too many files at %s\n",av[i]);
					command_errors++;
			}
			files++;
		}
	} /* for each command line argument */
}

/* open_files - opens the input and the output files */
static void open_files()
{
	int i;

	for (i=1; i<3; i++)
		if ((infile[i] = fopen (infile_name[i], "r")) == NULL) {
			printf ("Error: can't open %s\n", infile_name[i]);
			command_errors++;
		}
	if (files>2)
		if ((outfile = fopen (outfile_name, "w")) == NULL) {
			printf ("Error: can't create %s\n", outfile_name);
			command_errors++;
		}
}

/* help - outputs usage information */
static void help()
{
	printf ("diff -- version 3.0 1/11/96\n\n");
	printf ("diff - text file differencer and change barrer\n"
		"usage: diff [option{option}] newfile oldfile [barfile]\n"
		"\n"
		"options:\n");
#ifdef TRACER_FUNCTIONS
	printf ("   -t   trace operation, default off\n");
#endif
	printf ("   -b n column of barfile for change bar, default 78\n"
		"   -h n lines at top of page to skip for headers, default 0\n"
		"   -f n lines at bottom of page to skip for footers, default = 0\n"
		"   -p n lines per page (embedded form feeds override) default = 0\n"
		"   -c   uppercase/lowercase is significant (default is off)\n"
		"   -r n lines that must match before files are considered synced\n"
		"        after differences are found. default = 3\n"
		"   -w   blank lines are considered significant (default is not)\n"
		"   -s   screen listing off (default is on)\n"
		"   -l   print only line numbers (default is off)\n"
		"   -n n pages in NEWFILE to skip before compare.  Also sets -o. default = 0\n"
		"   -o n pages in OLDFILE to skip before compare.  Must come after -n.\n"
		"        default = 0\n");
	exit (3);
}


static char *nextline(int f)
 {
  static char buf[MAXLINE+1];

  buf[sizeof(buf)-1] = 0;
  return(fgets(buf, MAXLINE, infile[f]));
 }

static void putbarline(char *text)
{
 if (output)
  fputs (text, outfile);
}

static void putline(char *text)
{
 fputs(text, stdout);
}

/*--------------------------------------------------------------------------*/
int main (argc, argv)
	int argc;
	char *argv[];
{
	int result;
	extern int diff();

#if defined FORTIFY
        Fortify_EnterScope();
#endif

	if (argc == 1)
		help();

	strip_opt (argc, argv);
	if (files<2) {
		printf ("Error: Must specify two files\n");
		exit (2);
	}
	open_files();
	if (command_errors)
		exit (2);

	result = !diff(bar_col, top_skip, bot_skip, page_len, up_case, re_sync, blanks, skip1, skip2, onlylinenrs, trace_enabled, debug, nextline, (scr_on) ? putline : NULL, putbarline);
#if defined FORTIFY
	Fortify_LeaveScope();
#endif
	return(result);
}
